/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#include "curves/OE_subcurve.h"
#include <iostream>

#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <algorithm>

OE_subcurve::OE_subcurve(OE_curve* curve, float curveStart, float curveEnd, bool reverse)
{
	this->curve = curve;
	curve->addDependency(this);
	start = curveStart;
	end = curveEnd;
	this->reverse = reverse;
	refresh();
}


OE_subcurve::~OE_subcurve()
{
}


/** \brief check if the curve is valid.	 */
bool OE_subcurve::check()
{
	return curve;
}
		
/** \brief refresh the discPts array.	 */
void OE_subcurve::refresh()
{
	
	if (check()&&needRefresh)
	{
		curve->refresh();
	
		float tmpStart = start;
		float tmpEnd = end;
		if (start == -1) tmpStart= 0;
		if (end == -1) tmpEnd=(curve->getNpts()-1)/3;
		
		pts.clear();
		pts = curve->subCurve(tmpStart, tmpEnd, reverse);
	}
}

void OE_subcurve::delDependency(OE_base * object)
{
	if (curve == object) curve = 0;
}

bool OE_subcurve::setStart(float curveStart)
{
	start = curveStart;
	return true;
}

bool OE_subcurve::setEnd(float curveEnd)
{
	end = curveEnd;
	return true;
}