/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#include "stitchs/OE_birailstitch.h"
#include "curves/OE_pointcurve.h"
#include "curves/OE_curve.h"
#include <iostream>
#include <algorithm>

float OE_birailstitch::defaultLen = 1;
/*
OE_birailstitch::OE_birailstitch()
{
	this->len = defaultLen;
	this->maxLen = defaultMaxlen;
}
*/
OE_birailstitch::OE_birailstitch(OE_thread* thread, OE_curve* curve1,OE_curve* curve2, float len)
{
	this->curve1 = curve1;
	this->curve2 = curve2;
	curve1->addDependency(this);
	curve2->addDependency(this);
	this->len = len;
	this->maxLen = defaultMaxlen;
	this->thread = thread;
	
	refresh();
}

OE_birailstitch::OE_birailstitch(OE_thread* thread, OE_curve* curve1, OE_curve* curve2, bool reverse1, bool reverse2, float offset1, float offset2, float len)
{
	this->curve1 = curve1;
	curve1->addDependency(this);
	this->reverse1 = reverse1;
	this->offset1=offset1;
	
	this->curve2 = curve2;
	curve2->addDependency(this);
	this->reverse2 = reverse2;
	this->offset2=offset2;
	
	this->thread = thread;
	
	this->len = len;
	refresh();
}


OE_birailstitch::~OE_birailstitch()
{
}

void  OE_birailstitch::setCurve1(OE_curve* curve)
{
	if (this->curve1)this->curve1->removeDependency(this);
	this->curve1 = curve;
	curve->addDependency(this);
	setNeedRefresh();
}
void  OE_birailstitch::setReverse1(bool reverse){reverse1=reverse; setNeedRefresh();}
void  OE_birailstitch::setOffset1(float offset){offset1=offset; setNeedRefresh();}
void  OE_birailstitch::setCurve2(OE_curve* curve)
{
	if (this->curve2)this->curve2->removeDependency(this);
	this->curve2 = curve;
	curve->addDependency(this);
	setNeedRefresh();
}
void  OE_birailstitch::setReverse2(bool reverse){reverse2=reverse; setNeedRefresh();}
void  OE_birailstitch::setOffset2(float offset){offset2=offset; setNeedRefresh();}
void  OE_birailstitch::setLen(float len){this->len = len; setNeedRefresh();}


bool OE_birailstitch::check(){return (curve1&&curve2&&len);}

void OE_birailstitch::refresh()
{
	
	if (check()&&needRefresh)
	{
		curve1->refresh();
		curve2->refresh();
		
		std::vector<vector_2d> curve1Pts;
		std::vector<vector_2d> curve2Pts;
		
		if (curve1&&curve2&&(curve1->getNpts()>3)&&(curve2->getNpts()>3))
		{
			float curve1len = curve1->getLength(0.001);
			float curve2len = curve2->getLength(0.001);
			float medlen = (curve1len+curve2len)/2;
			
			if (curve1len==0||curve2len==0) return;
			
			curve1Pts = curve1->discretizeRegular(len*(curve1len/medlen));
			curve2Pts = curve2->discretizeRegular(len*(curve2len/medlen));
			
			if (reverse1) std::reverse(curve1Pts.begin(),curve1Pts.end());
			if (reverse2) std::reverse(curve2Pts.begin(),curve2Pts.end());

	
			if ((curve1Pts.size()<2)||(curve2Pts.size()<2)||(curve1Pts.size()!=curve2Pts.size()))
			{
				return;
			}
			
			//calcul des normales
			
			std::vector<vector_2d> normal1Pts = normals(curve1Pts);
			std::vector<vector_2d> normal2Pts = normals(curve2Pts);
			
			std::vector<vector_2d> 	outPts;
			vector_2d  vec1, vec2;
			for(unsigned i=0; i<curve1Pts.size()-1; i++)
			{
				outPts.push_back(curve1Pts.at(i)+ normal1Pts.at(i)*offset1);
				outPts.push_back(curve2Pts.at(i)+ normal2Pts.at(i)*-offset2);
			}
			
			pts.clear();
			if(maxLen>0)
				pts = stitchMaxLen(outPts, maxLen);
			else
				pts = outPts;
		}
		needRefresh = false;
	}
}

void OE_birailstitch::delDependency(OE_base * object)
{
	if (curve1 == object)	curve1 = 0;
	if (curve2 == object)	curve2 = 0;
}